@@ -47,6 +47,10 @@ except ImportError:
 
 from autobahn.websocket.types import ConnectionDeny
 
+if hasattr(asyncio, 'ensure_future'):
+    ensure_future = asyncio.ensure_future
+else:  # Deprecated since Python 3.4.4
+    ensure_future = asyncio.async
 
 __all__ = (
     'WebSocketAdapterProtocol',
@@ -127,64 +131,64 @@ class WebSocketAdapterProtocol(asyncio.Protocol):
     def _onOpen(self):
         res = self.onOpen()
         if yields(res):
-            asyncio.async(res)
+            ensure_future(res)
 
     def _onMessageBegin(self, isBinary):
         res = self.onMessageBegin(isBinary)
         if yields(res):
-            asyncio.async(res)
+            ensure_future(res)
 
     def _onMessageFrameBegin(self, length):
         res = self.onMessageFrameBegin(length)
         if yields(res):
-            asyncio.async(res)
+            ensure_future(res)
 
     def _onMessageFrameData(self, payload):
         res = self.onMessageFrameData(payload)
         if yields(res):
-            asyncio.async(res)
+            ensure_future(res)
 
     def _onMessageFrameEnd(self):
         res = self.onMessageFrameEnd()
         if yields(res):
-            asyncio.async(res)
+            ensure_future(res)
 
     def _onMessageFrame(self, payload):
         res = self.onMessageFrame(payload)
         if yields(res):
-            asyncio.async(res)
+            ensure_future(res)
 
     def _onMessageEnd(self):
         res = self.onMessageEnd()
         if yields(res):
-            asyncio.async(res)
+            ensure_future(res)
 
     def _onMessage(self, payload, isBinary):
         res = self.onMessage(payload, isBinary)
         if yields(res):
-            asyncio.async(res)
+            ensure_future(res)
 
     def _onPing(self, payload):
         res = self.onPing(payload)
         if yields(res):
-            asyncio.async(res)
+            ensure_future(res)
 
     def _onPong(self, payload):
         res = self.onPong(payload)
         if yields(res):
-            asyncio.async(res)
+            ensure_future(res)
 
     def _onClose(self, wasClean, code, reason):
         res = self.onClose(wasClean, code, reason)
         if yields(res):
-            asyncio.async(res)
+            ensure_future(res)
 
     def get_channel_id(self):
         """
         Implements :func:`autobahn.wamp.interfaces.ITransport.get_channel_id`
         """
-        # FIXME
-        raise Exception("transport channel binding not implemented for asyncio")
+        self.log.debug('FIXME: transport channel binding not implemented for asyncio (autobahn-python issue #729)')
+        return None
 
     def registerProducer(self, producer, streaming):
         raise Exception("not implemented")
@@ -202,14 +206,19 @@ class WebSocketServerProtocol(WebSocketAdapterProtocol, protocol.WebSocketServer
         # noinspection PyBroadException
         try:
             res = self.onConnect(request)
-            if yields(res):
-                asyncio.async(res)
         except ConnectionDeny as e:
             self.failHandshake(e.reason, e.code)
         except Exception as e:
             self.failHandshake("Internal server error: {}".format(e), ConnectionDeny.INTERNAL_SERVER_ERROR)
         else:
-            self.succeedHandshake(res)
+            if yields(res):
+                # if onConnect was an async method, we need to await
+                # the actual result before calling succeedHandshake
+                ensure_future(res).add_done_callback(
+                    lambda res: self.succeedHandshake(res.result())
+                )
+            else:
+                self.succeedHandshake(res)
 
 
 class WebSocketClientProtocol(WebSocketAdapterProtocol, protocol.WebSocketClientProtocol):
@@ -220,7 +229,7 @@ class WebSocketClientProtocol(WebSocketAdapterProtocol, protocol.WebSocketClient
     def _onConnect(self, response):
         res = self.onConnect(response)
         if yields(res):
-            asyncio.async(res)
+            ensure_future(res)
 
     def startTLS(self):
         raise Exception("WSS over explicit proxies not implemented")
