@@ -262,6 +262,38 @@ class TestSubscribeMessage(unittest.TestCase):
         self.assertEqual(msg.match, message.Subscribe.MATCH_PREFIX)
         self.assertEqual(msg.marshal(), wmsg)
 
+    def test_get_retained_default_false(self):
+        wmsg = [message.Subscribe.MESSAGE_TYPE, 123456, {u'match': u'prefix'}, u'com.myapp.topic1']
+        msg = message.Subscribe.parse(wmsg)
+        self.assertIsInstance(msg, message.Subscribe)
+        self.assertEqual(msg.request, 123456)
+        self.assertEqual(msg.topic, u'com.myapp.topic1')
+        self.assertEqual(msg.get_retained, None)
+        self.assertNotEqual(msg.get_retained, True)
+        self.assertEqual(msg.match, message.Subscribe.MATCH_PREFIX)
+        self.assertEqual(msg.marshal(), wmsg)
+
+    def test_get_retained_explicit_false(self):
+        wmsg = [message.Subscribe.MESSAGE_TYPE, 123456, {u'match': u'prefix', u'get_retained': False}, u'com.myapp.topic1']
+        msg = message.Subscribe.parse(wmsg)
+        self.assertIsInstance(msg, message.Subscribe)
+        self.assertEqual(msg.request, 123456)
+        self.assertEqual(msg.topic, u'com.myapp.topic1')
+        self.assertEqual(msg.get_retained, False)
+        self.assertNotEqual(msg.get_retained, True)
+        self.assertEqual(msg.match, message.Subscribe.MATCH_PREFIX)
+        self.assertEqual(msg.marshal(), wmsg)
+
+    def test_get_retained_explicit_true(self):
+        wmsg = [message.Subscribe.MESSAGE_TYPE, 123456, {u'match': u'prefix', u'get_retained': True}, u'com.myapp.topic1']
+        msg = message.Subscribe.parse(wmsg)
+        self.assertIsInstance(msg, message.Subscribe)
+        self.assertEqual(msg.request, 123456)
+        self.assertEqual(msg.topic, u'com.myapp.topic1')
+        self.assertEqual(msg.get_retained, True)
+        self.assertEqual(msg.match, message.Subscribe.MATCH_PREFIX)
+        self.assertEqual(msg.marshal(), wmsg)
+
 
 class TestSubscribedMessage(unittest.TestCase):
 
@@ -416,6 +448,39 @@ class TestPublishMessage(unittest.TestCase):
         self.assertEqual(msg.eligible, [100, 200, 300])
         self.assertEqual(msg.marshal(), wmsg)
 
+    def test_retain_default_false(self):
+        """
+        Retain, when not specified, is False-y by default.
+        """
+        wmsg = [message.Publish.MESSAGE_TYPE, 123456, {u'exclude_me': False, u'exclude': [300], u'eligible': [100, 200, 300]}, u'com.myapp.topic1']
+        msg = message.Publish.parse(wmsg)
+        self.assertIsInstance(msg, message.Publish)
+        self.assertEqual(msg.retain, None)
+        self.assertIsNot(msg.retain, True)
+        self.assertEqual(msg.marshal(), wmsg)
+
+    def test_retain_explicit_false(self):
+        """
+        Retain, when specified as False, shows up in the message.
+        """
+        wmsg = [message.Publish.MESSAGE_TYPE, 123456, {u'exclude_me': False, u'retain': False, u'exclude': [300], u'eligible': [100, 200, 300]}, u'com.myapp.topic1']
+        msg = message.Publish.parse(wmsg)
+        self.assertIsInstance(msg, message.Publish)
+        self.assertEqual(msg.retain, False)
+        self.assertIsNot(msg.retain, True)
+        self.assertEqual(msg.marshal(), wmsg)
+
+    def test_retain_explicit_true(self):
+        """
+        Retain, when specified as True, shows up in the message.
+        """
+        wmsg = [message.Publish.MESSAGE_TYPE, 123456, {u'exclude_me': False, u'retain': True, u'exclude': [300], u'eligible': [100, 200, 300]}, u'com.myapp.topic1']
+        msg = message.Publish.parse(wmsg)
+        self.assertIsInstance(msg, message.Publish)
+        self.assertEqual(msg.retain, True)
+        self.assertIs(msg.retain, True)
+        self.assertEqual(msg.marshal(), wmsg)
+
 
 class TestPublishedMessage(unittest.TestCase):
 
@@ -496,6 +561,29 @@ class TestEventMessage(unittest.TestCase):
         self.assertEqual(msg.publisher, 300)
         self.assertEqual(msg.marshal(), wmsg)
 
+    def test_retained_default_false(self):
+        wmsg = [message.Event.MESSAGE_TYPE, 123456, 789123, {}]
+        msg = message.Event.parse(wmsg)
+        self.assertIsInstance(msg, message.Event)
+        self.assertEqual(msg.retained, None)
+        self.assertNotEqual(msg.retained, True)
+        self.assertEqual(msg.marshal(), wmsg)
+
+    def test_retained_explicit_false(self):
+        wmsg = [message.Event.MESSAGE_TYPE, 123456, 789123, {u'retained': False}]
+        msg = message.Event.parse(wmsg)
+        self.assertIsInstance(msg, message.Event)
+        self.assertEqual(msg.retained, False)
+        self.assertNotEqual(msg.retained, True)
+        self.assertEqual(msg.marshal(), wmsg)
+
+    def test_retained_explicit_true(self):
+        wmsg = [message.Event.MESSAGE_TYPE, 123456, 789123, {u'retained': True}]
+        msg = message.Event.parse(wmsg)
+        self.assertIsInstance(msg, message.Event)
+        self.assertEqual(msg.retained, True)
+        self.assertEqual(msg.marshal(), wmsg)
+
 
 class TestRegisterMessage(unittest.TestCase):
 
@@ -936,12 +1024,20 @@ class TestHelloMessage(unittest.TestCase):
         self.assertEqual(msg[1], u"realm1")
         self.assertEqual(msg[2], {u'roles': {u'publisher': {u'features': {u'subscriber_blackwhite_listing': True}}}})
 
+        e = message.Hello(u"realm1", {u'publisher': role.RolePublisherFeatures(subscriber_blackwhite_listing=True)}, resumable=True)
+        msg = e.marshal()
+        self.assertEqual(len(msg), 3)
+        self.assertEqual(msg[0], message.Hello.MESSAGE_TYPE)
+        self.assertEqual(msg[1], u"realm1")
+        self.assertEqual(msg[2], {u'roles': {u'publisher': {u'features': {u'subscriber_blackwhite_listing': True}}}, u'resumable': True})
+
     def test_parse_and_marshal(self):
         wmsg = [message.Hello.MESSAGE_TYPE, u"realm1", {u'roles': {u'publisher': {}}}]
         msg = message.Hello.parse(wmsg)
         self.assertIsInstance(msg, message.Hello)
         self.assertEqual(msg.realm, u"realm1")
         self.assertEqual(msg.roles, {u'publisher': role.RolePublisherFeatures()})
+        self.assertEqual(msg.resumable, None)
         self.assertEqual(msg.marshal(), wmsg)
 
         wmsg = [message.Hello.MESSAGE_TYPE, u"realm1", {u'roles': {u'publisher': {u'features': {u'subscriber_blackwhite_listing': True}}}}]
@@ -951,6 +1047,24 @@ class TestHelloMessage(unittest.TestCase):
         self.assertEqual(msg.roles, {u'publisher': role.RolePublisherFeatures(subscriber_blackwhite_listing=True)})
         self.assertEqual(msg.marshal(), wmsg)
 
+        wmsg = [message.Hello.MESSAGE_TYPE, u"realm1", {u'roles': {u'publisher': {}}, u'resumable': False}]
+        msg = message.Hello.parse(wmsg)
+        self.assertIsInstance(msg, message.Hello)
+        self.assertEqual(msg.realm, u"realm1")
+        self.assertEqual(msg.roles, {u'publisher': role.RolePublisherFeatures()})
+        self.assertEqual(msg.resumable, False)
+        self.assertEqual(msg.marshal(), wmsg)
+
+        wmsg = [message.Hello.MESSAGE_TYPE, u"realm1", {u'roles': {u'publisher': {}}, u'resumable': True, u'resume-session': 1234, u'resume-token': u"dsjgsg"}]
+        msg = message.Hello.parse(wmsg)
+        self.assertIsInstance(msg, message.Hello)
+        self.assertEqual(msg.realm, u"realm1")
+        self.assertEqual(msg.roles, {u'publisher': role.RolePublisherFeatures()})
+        self.assertEqual(msg.resumable, True)
+        self.assertEqual(msg.resume_session, 1234)
+        self.assertEqual(msg.resume_token, u"dsjgsg")
+        self.assertEqual(msg.marshal(), wmsg)
+
     def test_str(self):
         e = message.Hello(u"realm1", {u'publisher': role.RolePublisherFeatures()})
         self.assertIsInstance(str(e), str)
